@@ -11,10 +11,11 @@ import time
 import shutil
 from .errors import ValidationError, RequestError
 from .crypto import *
+import tempfile
 
 
 class Mega(object):
-    def __init__(self):
+    def __init__(self, options=None):
         self.schema = 'https'
         self.domain = 'mega.co.nz'
         self.timeout = 160  # max time (secs) to wait for resp from api requests
@@ -22,11 +23,14 @@ class Mega(object):
         self.sequence_num = random.randint(0, 0xFFFFFFFF)
         self.request_id = make_id(10)
 
-    @classmethod
-    def login(cls, email, password):
-        instance = cls()
-        instance.login_user(email, password)
-        return instance
+        if options is None:
+            options = {}
+        self.options = options
+
+
+    def login(self, email, password):
+        self.login_user(email, password)
+        return self
 
     def login_user(self, email, password):
         password_aes = prepare_key(str_to_a32(password))
@@ -106,7 +110,7 @@ class Mega(object):
         Process a file
         """
         if file['t'] == 0 or file['t'] == 1:
-            keys = dict(keypart.split(':', 1) for keypart in file['k'].split('/'))
+            keys = dict(keypart.split(':', 1) for keypart in file['k'].split('/') if ':' in keypart)
             uid = file['u']
             key = None
             # my objects
@@ -289,6 +293,29 @@ class Mega(object):
         #convert bytes to megabyes
         return json_resp['mstrg'] / 1048576
 
+    def get_storage_space(self, giga=False, mega=False, kilo=False):
+        """
+        Get the current storage space.
+        Return a dict containing at least:
+          'used' : the used space on the account
+          'total' : the maximum space allowed with current plan
+        All storage space are in bytes unless asked differently.
+        """
+        if sum(1 if x else 0 for x in (kilo, mega, giga)) > 1:
+            raise ValueError("Only one unit prefix can be specified")
+        unit_coef = 1
+        if kilo:
+          unit_coef = 1024
+        if mega:
+          unit_coef = 1048576
+        if giga:
+          unit_coef = 1073741824
+        json_resp = self.api_request({'a': 'uq', 'xfer': 1, 'strg': 1})
+        return {
+            'used': json_resp['cstrg'] / unit_coef,
+            'total': json_resp['mstrg'] / unit_coef,
+            }
+
     def get_balance(self):
         """
         Get account monetary balance, Pro accounts only
@@ -394,7 +421,7 @@ class Mega(object):
             meta_mac = file['meta_mac']
 
         # Seems to happens sometime... When  this occurs, files are 
-        # inaccessible also in the official also in the official webapp.
+        # inaccessible also in the official also in the official web app.
         # Strangely, files can come back later.
         if 'g' not in file_data:
             raise RequestError('File not accessible anymore')
@@ -403,14 +430,15 @@ class Mega(object):
         attribs = base64_url_decode(file_data['at'])
         attribs = decrypt_attr(attribs, k)
         file_name = attribs['n']
-        file_name_tmp = '.megapy-%s-%s' % (int(time.time() * 1000), file_name)
 
         input_file = requests.get(file_url, stream=True).raw
 
-        if dest_path:
-            output_file = open(dest_path + '/' + file_name_tmp, 'wb')
+        if dest_path is None:
+            dest_path = ''
         else:
-            output_file = open(file_name_tmp, 'wb')
+            dest_path += '/'
+
+        temp_output_file = tempfile.NamedTemporaryFile(mode='w+b', prefix='megapy_', delete=False)
 
         counter = Counter.new(
             128, initial_value=((iv[0] << 32) + iv[1]) << 64)
@@ -420,7 +448,7 @@ class Mega(object):
         for chunk_start, chunk_size in sorted(get_chunks(file_size).items()):
             chunk = input_file.read(chunk_size)
             chunk = aes.decrypt(chunk)
-            output_file.write(chunk)
+            temp_output_file.write(chunk)
 
             chunk_mac = [iv[0], iv[1], iv[0], iv[1]]
             for i in range(0, len(chunk), 16):
@@ -441,13 +469,19 @@ class Mega(object):
                 file_mac[2] ^ chunk_mac[2],
                 file_mac[3] ^ chunk_mac[3]]
             file_mac = aes_cbc_encrypt_a32(file_mac, k)
-        output_file.close()
+
+            if self.options.get('verbose') is True:
+                # temp file size
+                file_info = os.stat(temp_output_file.name)
+                print('{0} of {1} downloaded'.format(file_info.st_size, file_size))
+
+        temp_output_file.close()
 
         # check mac integrity
         if (file_mac[0] ^ file_mac[1], file_mac[2] ^ file_mac[3]) != meta_mac:
             raise ValueError('Mismatched mac')
 
-        shutil.move(file_name_tmp, file_name)
+        shutil.move(temp_output_file.name, dest_path + file_name)
 
     ##########################################################################
     # UPLOAD
@@ -461,8 +495,8 @@ class Mega(object):
 
         #request upload url, call 'u' method
         input_file = open(filename, 'rb')
-        size = os.path.getsize(filename)
-        ul_url = self.api_request({'a': 'u', 's': size})['p']
+        file_size = os.path.getsize(filename)
+        ul_url = self.api_request({'a': 'u', 's': file_size})['p']
 
         #generate random aes key (128) for file
         ul_key = [random.randint(0, 0xFFFFFFFF) for _ in range(6)]
@@ -470,8 +504,10 @@ class Mega(object):
         aes = AES.new(a32_to_str(ul_key[:4]), AES.MODE_CTR, counter=count)
 
         file_mac = [0, 0, 0, 0]
-        for chunk_start, chunk_size in sorted(get_chunks(size).items()):
+        upload_progress = 0
+        for chunk_start, chunk_size in sorted(get_chunks(file_size).items()):
             chunk = input_file.read(chunk_size)
+            upload_progress += len(chunk)
 
             #determine chunks mac
             chunk_mac = [ul_key[4], ul_key[5], ul_key[4], ul_key[5]]
@@ -497,6 +533,10 @@ class Mega(object):
                                         data=chunk, timeout=self.timeout)
             completion_file_handle = output_file.text
 
+            if self.options.get('verbose') is True:
+                # upload progress
+                print('{0} of {1} uploaded'.format(upload_progress, file_size))
+
         #determine meta mac
         meta_mac = (file_mac[0] ^ file_mac[1], file_mac[2] ^ file_mac[3])
 
